/*
 *  Part of rvminis source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "helpfunc.h"
#include "instance.h"








RVInstance::RVInstance(void)
{
  poly=NULL;
  ncp_type=NULL;
  ncp_surface=NULL;
  vert=NULL;
}

RVInstance::~RVInstance(void)
{
  free_all();
}


bool RVInstance::exists(void)
{
  if(poly==NULL)
    return false;
  if(ncp_type==NULL)
    return false;
  if(ncp_surface==NULL)
    return false;
  if(vert==NULL)
    return false;
  return true;
}

void RVInstance::free_all(void)
{
  if(poly!=NULL)
    delete poly;
  if(ncp_type!=NULL)
    delete ncp_type;
  if(ncp_surface!=NULL)
    delete ncp_surface;
  if(vert!=NULL)
    delete vert;
}

void RVInstance::alloc_all(void)
{
  free_all();
  poly=new S_RV_Poly[header.num_polys];
  ncp_type=new rvlong[header.num_polys];
  ncp_surface=new rvlong[header.num_polys];
  vert=new S_RV_Vertex[header.num_vecs];
}








bool RVInstance::load(const char *fname, FileType ft)
{
  FILE *f;
  int i;

  if(ft==Auto)
    {
      if(filehastype(fname,"prm"))
	ft=RV_PRM;
      else if(filehastype(fname,"i-p"))
	ft=IM_PRM;
      else
	{
	  printf("Error: File '%s' is no instance. (Neither *.prm nor *.i-p)\n",fname);
	  return false;
	};
    };
	  

  f=fopen(fname,"rb");
  if(f==NULL)
    {
      perror("Cant read file");
      return false;
    };


  /*** Overread IM fake header ***/

  if(ft==IM_PRM)
    {
      char zeroes[16];
      int i;
      bool ok;

      /* skip 16 bytes of zero */
      if(!safe_fread(zeroes,16,1,f,fname))
	return false;
      ok=true;

      for(i=0;i<16;i++)
	if(zeroes[i] != 0)
	  ok=false;

      if(!ok)
	printf("Warning: Intermediate file '%s' has improper header. May be a corrupt file.\n",
	       fname);
    };

  if(!safe_fread(&header, S_RV_PRM_MESH_SIZE, 1, f, fname))
    return false;

  alloc_all();

  /*** Read Polygons ***/
  for(i=0;i<header.num_polys;i++)
    {
      if(!safe_fread(&poly[i], S_RV_POLY_SIZE,1,f,fname))
	{ free_all(); return false; }
      
      if(ft == RV_PRM)
	ncp_type[i]=ncp_surface[i]=0;
      else
	{
	  if(!safe_fread(&ncp_type[i], sizeof(rvlong),1,f,fname))
	    { free_all(); return false; }
	  if(!safe_fread(&ncp_surface[i], sizeof(rvlong),1,f,fname))
	    { free_all(); return false; }
	};
    };


  /*** Read Vertex list ***/
  if(!safe_fread(vert, S_RV_VERTEX_SIZE, header.num_vecs, f, fname))
    { free_all(); return false; }


  /* Did it. */

  fclose(f);
  return true;
}





bool RVInstance::save(const char *fname, FileType ft)
{
  FILE *f;
  int i;

  if(ft==Auto)
    {
      char newname[200];

      strcpy_suffix(newname, fname, "prm");
      if(!save(newname, RV_PRM))
	return false;
     
      strcpy_suffix(newname, fname, "i-p");
      return save(newname, IM_PRM);
    };

	  
  /*** Real saving: ***/


  f=fopen(fname,"wb");
  if(f==NULL)
    {
      char msg[200];
      sprintf(msg,"Cant write file '%s'",fname);
      perror(msg);
      return false;
    };


  /*** Write IM fake header ***/

  if(ft==IM_PRM)
    {
      char zeroes[16];

      bzero(zeroes,16);
      fwrite(zeroes,16,1,f);
    };

  fwrite(&header, S_RV_PRM_MESH_SIZE, 1, f);



  /*** Write Polygons ***/
  for(i=0;i<header.num_polys;i++)
    {
      fwrite(&poly[i], S_RV_POLY_SIZE,1,f);

      if(ft == IM_PRM)
	{
	  fwrite(&ncp_type[i], sizeof(rvlong),1,f);
	  fwrite(&ncp_surface[i], sizeof(rvlong),1,f);
	};
    };


  /*** Write Vertex list ***/
  fwrite(vert, S_RV_VERTEX_SIZE, header.num_vecs, f);

  fclose(f);
  return true;
}

void RVInstance::apply_modifier(poly_modifier pm)
{
  int i;

  for(i=0;i<header.num_polys;i++)
    {
      pm(&poly[i],&ncp_type[i],&ncp_surface[i],vert);
    };
}
