/*
 *  Part of rvminis source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __RVTYPES_H
#define __RVTYPES_H

#include <math.h>

typedef float                rvfloat;   /* 32 bit */
typedef          short int   rvshort;   /* 16 bit */
typedef unsigned short int   rvushort;  /* 16 bit */
typedef                int   rvlong;    /* 32 bit */
typedef unsigned       int   rvulong;   /* 32 bit */



#define SQR(f) ((f)*(f))

#define EPSILON (0.0001)

#define FEQUAL(a,b)   ( fabs((a)-(b))<EPSILON )





struct S_RV_UV
{
  rvfloat     u; // Texture coords: 0<u,v<1
  rvfloat     v;
};
#define S_RV_UV_SIZE sizeof(S_RV_UV)





/*
 * S_RV_Vector
 *
 * simply three floats (x, y and z)
 *
 * class is 'Vector' (vector.h)
 */
struct S_RV_Vector
{
  S_RV_Vector& operator+=(const S_RV_Vector& vec)
    {
      coord[0]+=vec.coord[0];
      coord[1]+=vec.coord[1];
      coord[2]+=vec.coord[2];
      return *this;
    };

  rvfloat     coord[3];
};
#define S_RV_VECTOR_SIZE sizeof(S_RV_Vector)




struct S_RV_W_Mesh
{
  S_RV_Vector   ball_center;
  rvfloat     ball_radius;

  rvfloat     bbox_xlo, bbox_xhi;
  rvfloat     bbox_ylo, bbox_yhi;
  rvfloat     bbox_zlo, bbox_zhi;

  rvushort    num_polys;
  rvushort    num_vecs;
};
#define S_RV_W_MESH_SIZE sizeof(S_RV_W_Mesh)



struct S_RV_PRM_Mesh
{
  rvushort    num_polys;
  rvushort    num_vecs;
};
#define S_RV_PRM_MESH_SIZE sizeof(S_RV_PRM_Mesh)









/*
 * S_RV_Poly
 *
 * A three- or four-sided, flat polygon
 * with a texture.
 *
 * class is 'Polygon' (polygon.h)
 */
struct S_RV_Poly
{
  rvushort    type; // bitfield

  rvshort     texture; // 0=a.bmp, 1=b.bmp,..., -1=Solid colored

  rvushort    vertidx[4];
  rvulong     color[4];
  S_RV_UV     texcoord[4];
};
#define S_RV_POLY_SIZE sizeof(S_RV_Poly)






/*
 * S_RV_Vertex
 *
 * Two vectors: 
 *              'position' (x,y,z)
 *              'normal'   (u,v,w)
 *
 * class is 'Vertex' (vertex.h)
 */
struct S_RV_Vertex
{
  S_RV_Vector   pos;
  S_RV_Vector   normal;
};
#define S_RV_VERTEX_SIZE sizeof(S_RV_Vertex)




struct S_RV_FunnyBall
{
  S_RV_Vector   center;
  rvfloat     radius;
  
  rvulong     groupcnt;
};
#define S_RV_FUNNYBALL_SIZE sizeof(S_RV_FunnyBall)






struct S_RV_Plane
{
  S_RV_Vector  normal;
  rvfloat    distance;
};
#define S_RV_PLANE_SIZE sizeof(S_RV_Plane);




struct S_RV_Polyhedron
{
  rvlong   type;
  rvlong   surface;

  S_RV_Plane plane[5];

  rvfloat  bbox_xlo, bbox_xhi;
  rvfloat  bbox_ylo, bbox_yhi;
  rvfloat  bbox_zlo, bbox_zhi;
};
#define S_RV_POLYHEDRON_SIZE sizeof(S_RV_Polyhedron)




struct S_RV_LookupGrid
{
  rvfloat    x0;
  rvfloat    z0;

  rvfloat    x_size;
  rvfloat    z_size;

  rvfloat    raster_size;
};
#define S_RV_LOOKUPGRID_SIZE sizeof(S_RV_LookupGrid)







struct S_FIN_Data
{
  char fname[9];

  unsigned char rgb_model[3];
  unsigned char rgb_env[3];

  unsigned char _unknown[5];

  rvlong lodbias;

  S_RV_Vector position;
  rvfloat rot[3][3];
};
#define S_FIN_DATA_SIZE sizeof(S_FIN_Data)








struct S_PAN_Header  // 10/04/2002 vag
{
  rvushort      cnt;
  unsigned char _unknown[9];
};
#define S_PAN_HEADER_SIZE sizeof(S_PAN_Header)


struct S_PAN_Data   // 10/04/2002 vag
{
  S_RV_Vector   position;
  unsigned char _unknown[36];
};
#define S_PAN_DATA_SIZE sizeof(S_PAN_Data)








struct S_FAN_Header // 10/04/2002 vag
{
  rvushort      cnt;
  unsigned char _unknown[9];
};
#define S_FAN_HEADER_SIZE sizeof(S_FAN_Header)


struct S_FAN_Data   // 10/04/2002 vag
{
  unsigned char _unknown[36];
  unsigned char _unknownFlags1[4];
  S_RV_Vector   pos1;
  unsigned char _unknownFlags2[4];
  S_RV_Vector   pos2;
  unsigned char _unknownTrailer[8];
};
#define S_FAN_DATA_SIZE sizeof(S_FAN_Data)

#endif
